from itertools import takewhile
from typing import Iterable

import pandas as pd
import seaborn as sns
from matplotlib import pyplot as plt

from lazy_budget.budget import Budget, FinancialOperation


def burndown_chart(budget: Budget, operations: Iterable[FinancialOperation]):
    max_date = budget.today
    operations = budget.filter(operations)
    spent_per_day = budget.get_spent_by_day(operations)
    for_df = []
    total_spent = budget.total.amount
    for day in takewhile(lambda x: x <= max_date, iter(budget)):
        budget.today = day
        ops = budget.filter(operations)
        total_spent += spent_per_day[day].amount
        should_be_available = (
            budget.days_left * budget.available_per_day.amount + budget.keep.amount
        )
        for_df.append(
            (
                day.strftime("%d"),
                total_spent,
                should_be_available,
                0,
                budget.available_per_day.amount,
                budget.get_currently_keeping(ops).amount,
                budget.keep.amount,
            )
        )
    df = pd.DataFrame(
        for_df,
        columns=[
            "day",
            "money",
            "should_be_available",
            "zero",
            "available_per_day",
            "currently_keeping",
            "keep",
        ],
    )
    sns.set_theme()
    sns.set_style("darkgrid")

    plt.plot("day", "money", data=df, color="red", label="real burndown")
    plt.plot(
        "day",
        "should_be_available",
        data=df,
        linestyle="dashed",
        color="red",
        label="planned burndown",
    )
    # plt.plot("day", "zero", data=df, linestyle="dashed")
    plt.plot("day", "currently_keeping", data=df, color="green", label="real savings")
    plt.plot(
        "day", "keep", data=df, color="green", linestyle=":", label="planned savings"
    )
    plt.legend()
    plt.show()



import os
from argparse import ArgumentParser
from dataclasses import dataclass
from datetime import date, datetime
from os import PathLike
from os.path import expanduser

import yaml

from lazy_budget.budget import Budget, BudgetStats, FinancialOperation
from lazy_budget.charts import burndown_chart
from lazy_budget.constants import (
    BUDGET_FILE,
    CONFIG_FILE_PATH,
    DEFAULT_LOCALE,
    HISTORY_FILE,
)
from lazy_budget.display import BasicStatsDisplay
from lazy_budget.money_provider import DEFAULT_CURRENCY, parse_money, str_to_currency


def do_nothing(args):
    print("specify a valid operation!")
    exit(2)


def _get_budget(args) -> Budget:
    return Budget.from_file(args.budget_file, args.currency, args.today)


def _get_history(args) -> FinancialOperation:
    return FinancialOperation.from_file(args.history_file, args.currency)


def add_new_operation(args):
    budget = _get_budget(args)
    op = FinancialOperation(
        money_value=parse_money(args.value, budget.currency),
        description=args.description,
        dttm=datetime.fromisoformat(args.dttm),
    )
    op.save(args.history_file)


def list_stats(args):
    budget = _get_budget(args)
    ops = _get_history(args)
    stats = BudgetStats.get_stats(budget, ops)
    display = BasicStatsDisplay(stats, locale=args.locale)
    print(display)


def show_burndown_chart(args):
    budget = _get_budget(args)
    ops = _get_history(args)
    burndown_chart(budget, ops)


@dataclass
class CLIConfig:
    history_file: PathLike = HISTORY_FILE
    budget_file: PathLike = BUDGET_FILE
    locale: str = DEFAULT_LOCALE
    currency: str = DEFAULT_CURRENCY.name

    def get_args(self, argv=None):
        parser = ArgumentParser()
        parser.add_argument(
            "-f",
            "--history-file",
            help=f"history file to use (default is {self.history_file})",
            default=self.history_file,
        )
        parser.add_argument(
            "-c",
            "--budget-file",
            help=f"budget file to use (default is {self.budget_file})",
            default=self.budget_file,
        )
        parser.add_argument(
            "-L",
            "--locale",
            help=f"locale to use (default is {self.locale})",
            default=self.locale,
        )
        parser.add_argument(
            "-C",
            "--currency",
            help=f"default currency to use in uncertain situations (default is {self.currency})",
            default=self.currency,
            type=str_to_currency,
        )
        parser.add_argument(
            "-t",
            "--today",
            help='a date used as "today" during stats, useful for seeing old budget info',
            type=date.fromisoformat,
            default=None,
        )

        parser.set_defaults(action_func=lambda args: None)
        subparsers = parser.add_subparsers(
            help="whether you want to add an expense or see statistics"
        )

        parser_add = subparsers.add_parser(
            "add", aliases=["a", "new", "n"], help="add a new financial operation"
        )
        parser_add.set_defaults(action_func=add_new_operation)
        parser_add.add_argument(
            "value",
            help='amount (and optionally currency) of financial operation, e.g. "-10.99 USD", "2.50", "+3.65"',
        )
        parser_add.add_argument("description")
        parser_add.add_argument(
            "dttm",
            nargs="?",
            default=datetime.now().isoformat(),
            help="date and time of the operation; now is default",
        )

        parser_today = subparsers.add_parser(
            "list-stats",
            aliases=["ls", "now", "today", "list", "stats", "s", "get"],
            help="list statistics",
        )
        parser_today.set_defaults(action_func=list_stats)

        parser_burndown = subparsers.add_parser(
            "burndown", aliases=["bd"], help="shows a burndown chart of the budget"
        )
        parser_burndown.set_defaults(action_func=show_burndown_chart)

        return parser.parse_args(argv)

    @classmethod
    def from_file(cls, filename: PathLike = None) -> "CLIConfig":
        if not filename:
            filename = os.getenv("LBUDRC") or expanduser(CONFIG_FILE_PATH)

        try:
            with open(filename, "r") as fp:
                data = yaml.safe_load(fp)
        except IOError:
            print(f"config file not found at {filename}, using the defaults")
            return cls()

        return cls(**data)
